@@ -9,9 +9,10 @@ typedef struct {
   OggVorbis_File *ovf;
 } py_vorbisfile;
 
+#define PY_VORBISFILE(x) (((py_vorbisfile *)x)->ovf)
 extern PyTypeObject py_vorbisfile_type;
 
-#endif // __PYVORBIS_FILE_H__
+#endif /* __PYVORBIS_FILE_H__ */
 
 
 
