@@ -10,20 +10,20 @@ ogg_module_info *modinfo;
 
 PyObject *Py_VorbisError;
 
-// Object docstrings
+/* Object docstrings */
 
 extern char py_vorbisfile_doc[];
 extern char py_vinfo_doc[];
 extern char py_vcomment_doc[];
 extern char py_dsp_doc[];
 
-// Module-accessible functions
+/* Module-accessible functions */
 
 PyObject *py_file_new(PyObject *, PyObject *);
 PyObject *py_info_new(PyObject *, PyObject *, PyObject *);
 PyObject *py_comment_new(PyObject *, PyObject *);
 
-// Utility functions/macros
+/* Utility functions/macros */
 
 PyObject *v_error_from_code(int, char*);
 
@@ -34,5 +34,5 @@ PyObject *v_error_from_code(int, char*);
                                    return Py_None;
 
 
-#endif // __VORBISMODULE_H__
+#endif /* __VORBISMODULE_H__ */
 
